#!/usr/bin/env python3

# pyXLMS - TESTS
# 2024 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

import pytest


XL_TSV = "data/xlinkx/XLpeplib_Beveridge_Lumos_DSSO_MS3_Crosslinks.txt"
XL_XLSX = "data/xlinkx/XLpeplib_Beveridge_Lumos_DSSO_MS3_Crosslinks.xlsx"
CSMS_TSV = "data/xlinkx/XLpeplib_Beveridge_Lumos_DSSO_MS3_CSMs.txt"
CSMS_XLSX = "data/xlinkx/XLpeplib_Beveridge_Lumos_DSSO_MS3_CSMs.xlsx"


def test1():
    from pyXLMS import parser as p

    parser_result = p.read_xlinkx(XL_TSV, parse_modifications=False, modifications={})
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)


def test2():
    from pyXLMS import parser as p

    parser_result = p.read_xlinkx(XL_XLSX, parse_modifications=False, modifications={})
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)


def test3():
    from pyXLMS import parser as p

    parser_result = p.read_xlinkx(
        [XL_TSV, XL_XLSX], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 472

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)


def test4():
    from pyXLMS import parser as p

    parser_result = p.read_xlinkx(
        [XL_TSV, XL_TSV], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 472

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)


def test5():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(CSMS_TSV, parse_modifications=False, modifications={})
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is None

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test6():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        CSMS_XLSX, parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is None

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test7():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [CSMS_TSV, CSMS_XLSX], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is None

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823 * 2

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test8():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [CSMS_TSV, CSMS_TSV], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "partial"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is None

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823 * 2

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test9():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [XL_XLSX, CSMS_XLSX], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "full"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test10():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [XL_XLSX, CSMS_TSV], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "full"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test11():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [XL_TSV, CSMS_XLSX], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "full"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test12():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [XL_TSV, CSMS_TSV], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "full"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None


def test13():
    from pyXLMS import parser as p
    from pyXLMS.transform import modifications_to_str as mts

    parser_result = p.read_xlinkx(
        [XL_XLSX, CSMS_XLSX, CSMS_TSV], parse_modifications=False, modifications={}
    )
    assert parser_result["data_type"] == "parser_result"
    assert parser_result["completeness"] == "full"
    assert parser_result["search_engine"] == "XlinkX"
    assert parser_result["crosslink-spectrum-matches"] is not None
    assert parser_result["crosslinks"] is not None

    crosslinks = parser_result["crosslinks"]
    assert len(crosslinks) == 236

    first_crosslink = crosslinks[0]
    last_crosslink = crosslinks[-1]

    assert first_crosslink["data_type"] == "crosslink"
    assert first_crosslink["completeness"] == "full"
    assert first_crosslink["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert first_crosslink["alpha_peptide_crosslink_position"] == 1
    assert first_crosslink["alpha_proteins"] == ["Cas9"]
    assert first_crosslink["alpha_proteins_crosslink_positions"] == [49]
    assert not first_crosslink["alpha_decoy"]
    assert first_crosslink["beta_peptide"] == "LVDSTDKADLR"
    assert first_crosslink["beta_peptide_crosslink_position"] == 7
    assert first_crosslink["beta_proteins"] == ["Cas9"]
    assert first_crosslink["beta_proteins_crosslink_positions"] == [152]
    assert not first_crosslink["beta_decoy"]
    assert first_crosslink["crosslink_type"] == "intra"
    assert first_crosslink["score"] == pytest.approx(445.6)

    assert last_crosslink["data_type"] == "crosslink"
    assert last_crosslink["completeness"] == "full"
    assert last_crosslink["alpha_peptide"] == "LESEFVYGDYKVYDVR"
    assert last_crosslink["alpha_peptide_crosslink_position"] == 11
    assert last_crosslink["alpha_proteins"] == ["Cas9"]
    assert last_crosslink["alpha_proteins_crosslink_positions"] == [1018]
    assert not last_crosslink["alpha_decoy"]
    assert last_crosslink["beta_peptide"] == "VVDELVKVMGR"
    assert last_crosslink["beta_peptide_crosslink_position"] == 7
    assert last_crosslink["beta_proteins"] == ["Cas9"]
    assert last_crosslink["beta_proteins_crosslink_positions"] == [753]
    assert not last_crosslink["beta_decoy"]
    assert last_crosslink["crosslink_type"] == "intra"
    assert last_crosslink["score"] == pytest.approx(11.49)

    csms = parser_result["crosslink-spectrum-matches"]
    assert len(csms) == 823 * 2

    first_csm = csms[0]
    csm = csms[21]
    last_csm = csms[-1]

    assert first_csm["data_type"] == "crosslink-spectrum-match"
    assert first_csm["completeness"] == "partial"
    assert first_csm["alpha_peptide"] == "KNLIGALLFDSGETAEATR"
    assert mts(first_csm["alpha_modifications"]) is None
    assert first_csm["alpha_peptide_crosslink_position"] == 1
    assert first_csm["alpha_proteins"] == ["Cas9"]
    assert first_csm["alpha_proteins_crosslink_positions"] == [49]
    assert first_csm["alpha_proteins_peptide_positions"] == [49]
    assert first_csm["alpha_score"] is None
    assert not first_csm["alpha_decoy"]
    assert first_csm["beta_peptide"] == "LVDSTDKADLR"
    assert mts(first_csm["beta_modifications"]) is None
    assert first_csm["beta_peptide_crosslink_position"] == 7
    assert first_csm["beta_proteins"] == ["Cas9"]
    assert first_csm["beta_proteins_crosslink_positions"] == [152]
    assert first_csm["beta_proteins_peptide_positions"] == [146]
    assert first_csm["beta_score"] is None
    assert not first_csm["beta_decoy"]
    assert first_csm["crosslink_type"] == "intra"
    assert first_csm["score"] == pytest.approx(445.6)
    assert first_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert first_csm["scan_nr"] == 43526
    assert first_csm["charge"] == 3
    assert first_csm["retention_time"] == pytest.approx(88.612 * 60.0)
    assert first_csm["ion_mobility"] is None

    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "NLGKVGSKCCK"
    assert mts(csm["alpha_modifications"]) is None
    assert csm["alpha_peptide_crosslink_position"] == 8
    assert csm["alpha_proteins"] == ["spALBU_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [460]
    assert csm["alpha_proteins_peptide_positions"] == [453]
    assert csm["alpha_score"] is None
    assert csm["alpha_decoy"] is None
    assert csm["beta_peptide"] == "TILDFLKSDGFANR"
    assert mts(csm["beta_modifications"]) is None
    assert csm["beta_peptide_crosslink_position"] == 7
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [688]
    assert csm["beta_proteins_peptide_positions"] == [682]
    assert csm["beta_score"] is None
    assert csm["beta_decoy"] is None
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(297.32)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert csm["scan_nr"] == 46493
    assert csm["charge"] == 3
    assert csm["retention_time"] == pytest.approx(94.285 * 60.0)
    assert csm["ion_mobility"] is None

    assert last_csm["data_type"] == "crosslink-spectrum-match"
    assert last_csm["completeness"] == "partial"
    assert last_csm["alpha_peptide"] == "ESILPKR"
    assert mts(last_csm["alpha_modifications"]) is None
    assert last_csm["alpha_peptide_crosslink_position"] == 6
    assert last_csm["alpha_proteins"] == ["Cas9"]
    assert last_csm["alpha_proteins_crosslink_positions"] == [1117]
    assert last_csm["alpha_proteins_peptide_positions"] == [1112]
    assert last_csm["alpha_score"] is None
    assert last_csm["alpha_decoy"] is None
    assert last_csm["beta_peptide"] == "KMIAK"
    assert mts(last_csm["beta_modifications"]) is None
    assert last_csm["beta_peptide_crosslink_position"] == 1
    assert last_csm["beta_proteins"] == ["Cas9"]
    assert last_csm["beta_proteins_crosslink_positions"] == [1024]
    assert last_csm["beta_proteins_peptide_positions"] == [1024]
    assert last_csm["beta_score"] is None
    assert last_csm["beta_decoy"] is None
    assert last_csm["crosslink_type"] == "intra"
    assert last_csm["score"] == pytest.approx(7.45)
    assert last_csm["spectrum_file"] == "XLpeplib_Beveridge_Lumos_DSSO_MS3.raw"
    assert last_csm["scan_nr"] == 13466
    assert last_csm["charge"] == 4
    assert last_csm["retention_time"] == pytest.approx(36.2167 * 60.0)
    assert last_csm["ion_mobility"] is None



#!/usr/bin/env python3

# pyXLMS - TESTS
# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

import pytest

MAXQUANT1 = "data/maxquant/run1/crosslinkMsms.txt"


def test1():
    from pyXLMS.parser import read_maxquant
    from pyXLMS.transform import modifications_to_str as mts

    with open(MAXQUANT1, "r", encoding="utf-8") as f:
        pr = read_maxquant(f, crosslinker="DSS")
        f.close()

    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MaxQuant"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 730

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GQKNSR"
    assert mts(csm["alpha_modifications"]) == "(3:[DSS|138.06808])"
    assert csm["alpha_peptide_crosslink_position"] == 3
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [779]
    assert csm["alpha_proteins_peptide_positions"] == [777]
    assert csm["alpha_score"] == pytest.approx(46.617672)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "GQKNSR"
    assert mts(csm["beta_modifications"]) == "(3:[DSS|138.06808])"
    assert csm["beta_peptide_crosslink_position"] == 3
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [779]
    assert csm["beta_proteins_peptide_positions"] == [777]
    assert csm["beta_score"] == pytest.approx(46.617672)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(46.618)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 2257
    assert csm["charge"] == 3
    assert csm["retention_time"] is None
    assert csm["ion_mobility"] is None

    csm = csms[593]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "DFLELKANMAGQADAGFDGPK"
    assert mts(csm["alpha_modifications"]) == "(6:[DSS|138.06808])"
    assert csm["alpha_peptide_crosslink_position"] == 6
    assert csm["alpha_proteins"] == ["sp|MYG_HUMAN|"]
    assert csm["alpha_proteins_crosslink_positions"] == [20]
    assert csm["alpha_proteins_peptide_positions"] == [15]
    assert csm["alpha_score"] == pytest.approx(16.773198)
    assert csm["alpha_decoy"]
    assert csm["beta_peptide"] == "YVPKIK"
    assert mts(csm["beta_modifications"]) == "(4:[DSS|138.06808])"
    assert csm["beta_peptide_crosslink_position"] == 4
    assert csm["beta_proteins"] == ["sp|MYG_HUMAN|"]
    assert csm["beta_proteins_crosslink_positions"] == [55]
    assert csm["beta_proteins_peptide_positions"] == [52]
    assert csm["beta_score"] == pytest.approx(22.066486)
    assert csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(13.746)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 17682
    assert csm["charge"] == 4
    assert csm["retention_time"] is None
    assert csm["ion_mobility"] is None

    csm = csms[723]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "MDGTEELLVKLNR"
    assert (
        mts(csm["alpha_modifications"])
        == "(1:[Oxidation|15.994915]);(10:[DSS|138.06808])"
    )
    assert csm["alpha_peptide_crosslink_position"] == 10
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [396]
    assert csm["alpha_proteins_peptide_positions"] == [387]
    assert csm["alpha_score"] == pytest.approx(121.292258)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "MDGTEELLVKLNR"
    assert mts(csm["beta_modifications"]) == "(10:[DSS|138.06808])"
    assert csm["beta_peptide_crosslink_position"] == 10
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [396]
    assert csm["beta_proteins_peptide_positions"] == [387]
    assert csm["beta_score"] == pytest.approx(140.526596)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(229.48)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 23071
    assert csm["charge"] == 4
    assert csm["retention_time"] is None
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "MLLDIKTR"
    assert mts(csm["alpha_modifications"]) == "(6:[DSS|138.06808])"
    assert csm["alpha_peptide_crosslink_position"] == 6
    assert csm["alpha_proteins"] == ["sp|K1C15_SHEEP|"]
    assert csm["alpha_proteins_crosslink_positions"] == [394]
    assert csm["alpha_proteins_peptide_positions"] == [389]
    assert csm["alpha_score"] == pytest.approx(26.076371)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "TEVQTGGFSKESILPK"
    assert mts(csm["beta_modifications"]) == "(10:[DSS|138.06808])"
    assert csm["beta_peptide_crosslink_position"] == 10
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [1111]
    assert csm["beta_proteins_peptide_positions"] == [1102]
    assert csm["beta_score"] == pytest.approx(11.355476)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "inter"
    assert csm["score"] == pytest.approx(23.711)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 23784
    assert csm["charge"] == 3
    assert csm["retention_time"] is None
    assert csm["ion_mobility"] is None



#!/usr/bin/env python3

# pyXLMS - TESTS
# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

import pytest

MEROX1 = "data/merox/XLpeplib_Beveridge_QEx-HFX_DSS_R1.csv"
MEROX2 = "data/merox/XLpeplib_Beveridge_QEx-HFX_DSS_R1.zhrm"
MEROX3 = "data/merox/cas11.csv"


def test1():
    from pyXLMS.parser import read_merox

    pr = read_merox(
        MEROX1, crosslinker="DSS", parse_modifications=False, modifications={}
    )
    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MeroX"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 93

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GKSDNVPSEEVVK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 2
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [870]
    assert csm["alpha_proteins_peptide_positions"] == [869]
    assert csm["alpha_score"] == pytest.approx(13.14747)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "VKYVTEGmR"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 2
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [532]
    assert csm["beta_proteins_peptide_positions"] == [531]
    assert csm["beta_score"] == pytest.approx(12.143336)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(97.0)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 10061
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(3099.0)
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "KLVDSTDK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 1
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [145]
    assert csm["alpha_proteins_peptide_positions"] == [145]
    assert csm["alpha_score"] == pytest.approx(77.979659)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "KLVDSTDK"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 1
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [145]
    assert csm["beta_proteins_peptide_positions"] == [145]
    assert csm["beta_score"] == pytest.approx(77.979659)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(139)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 8258
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(2605.0)
    assert csm["ion_mobility"] is None


def test2():
    from pyXLMS.parser import read_merox

    pr = read_merox(
        MEROX2, crosslinker="DSS", parse_modifications=False, modifications={}
    )
    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MeroX"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 93

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GKSDNVPSEEVVK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 2
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [870]
    assert csm["alpha_proteins_peptide_positions"] == [869]
    assert csm["alpha_score"] == pytest.approx(13.14747)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "VKYVTEGmR"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 2
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [532]
    assert csm["beta_proteins_peptide_positions"] == [531]
    assert csm["beta_score"] == pytest.approx(12.143336)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(97.0)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 10061
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(3099.0)
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "KLVDSTDK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 1
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [145]
    assert csm["alpha_proteins_peptide_positions"] == [145]
    assert csm["alpha_score"] == pytest.approx(77.979659)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "KLVDSTDK"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 1
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [145]
    assert csm["beta_proteins_peptide_positions"] == [145]
    assert csm["beta_score"] == pytest.approx(77.979659)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(139)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 8258
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(2605.0)
    assert csm["ion_mobility"] is None


def test3():
    from pyXLMS.parser import read_merox

    pr = read_merox(
        MEROX3, crosslinker="DSS", parse_modifications=False, modifications={}
    )
    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MeroX"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 93

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GKSDNVPSEEVVK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 2
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [870]
    assert csm["alpha_proteins_peptide_positions"] == [869]
    assert csm["alpha_score"] == pytest.approx(13.14747)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "VKYVTEGmR"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 2
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [532]
    assert csm["beta_proteins_peptide_positions"] == [531]
    assert csm["beta_score"] == pytest.approx(12.143336)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(97.0)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 10061
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(3099.0)
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "KLVDSTDK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 1
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [145]
    assert csm["alpha_proteins_peptide_positions"] == [145]
    assert csm["alpha_score"] == pytest.approx(77.979659)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "KLVDSTDK"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 1
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [145]
    assert csm["beta_proteins_peptide_positions"] == [145]
    assert csm["beta_score"] == pytest.approx(77.979659)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(139)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 8258
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(2605.0)
    assert csm["ion_mobility"] is None


def test4():
    from pyXLMS.parser import read_merox

    with open(MEROX1, "r", encoding="utf-8") as f:
        pr = read_merox(
            f, crosslinker="DSS", parse_modifications=False, modifications={}
        )
        f.close()

    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MeroX"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 93

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GKSDNVPSEEVVK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 2
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [870]
    assert csm["alpha_proteins_peptide_positions"] == [869]
    assert csm["alpha_score"] == pytest.approx(13.14747)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "VKYVTEGmR"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 2
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [532]
    assert csm["beta_proteins_peptide_positions"] == [531]
    assert csm["beta_score"] == pytest.approx(12.143336)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(97.0)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 10061
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(3099.0)
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "KLVDSTDK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 1
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [145]
    assert csm["alpha_proteins_peptide_positions"] == [145]
    assert csm["alpha_score"] == pytest.approx(77.979659)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "KLVDSTDK"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 1
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [145]
    assert csm["beta_proteins_peptide_positions"] == [145]
    assert csm["beta_score"] == pytest.approx(77.979659)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(139)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 8258
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(2605.0)
    assert csm["ion_mobility"] is None


def test5():
    from pyXLMS.parser import read_merox

    with open(MEROX2, "rb") as f:
        pr = read_merox(
            f, crosslinker="DSS", parse_modifications=False, modifications={}
        )
        f.close()

    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MeroX"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 93

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GKSDNVPSEEVVK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 2
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [870]
    assert csm["alpha_proteins_peptide_positions"] == [869]
    assert csm["alpha_score"] == pytest.approx(13.14747)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "VKYVTEGmR"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 2
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [532]
    assert csm["beta_proteins_peptide_positions"] == [531]
    assert csm["beta_score"] == pytest.approx(12.143336)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(97.0)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 10061
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(3099.0)
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "KLVDSTDK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 1
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [145]
    assert csm["alpha_proteins_peptide_positions"] == [145]
    assert csm["alpha_score"] == pytest.approx(77.979659)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "KLVDSTDK"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 1
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [145]
    assert csm["beta_proteins_peptide_positions"] == [145]
    assert csm["beta_score"] == pytest.approx(77.979659)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(139)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 8258
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(2605.0)
    assert csm["ion_mobility"] is None


def test6():
    from pyXLMS.parser import read_merox

    with open(MEROX3, "r", encoding="utf-8") as f:
        pr = read_merox(
            f, crosslinker="DSS", parse_modifications=False, modifications={}
        )
        f.close()

    assert pr["data_type"] == "parser_result"
    assert pr["completeness"] == "partial"
    assert pr["search_engine"] == "MeroX"
    assert pr["crosslink-spectrum-matches"] is not None
    assert pr["crosslinks"] is None

    csms = pr["crosslink-spectrum-matches"]
    assert len(csms) == 93

    csm = csms[0]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "GKSDNVPSEEVVK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 2
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [870]
    assert csm["alpha_proteins_peptide_positions"] == [869]
    assert csm["alpha_score"] == pytest.approx(13.14747)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "VKYVTEGmR"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 2
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [532]
    assert csm["beta_proteins_peptide_positions"] == [531]
    assert csm["beta_score"] == pytest.approx(12.143336)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(97.0)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 10061
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(3099.0)
    assert csm["ion_mobility"] is None

    csm = csms[-1]
    assert csm["data_type"] == "crosslink-spectrum-match"
    assert csm["completeness"] == "partial"
    assert csm["alpha_peptide"] == "KLVDSTDK"
    assert csm["alpha_modifications"] is None
    assert csm["alpha_peptide_crosslink_position"] == 1
    assert csm["alpha_proteins"] == ["Cas9"]
    assert csm["alpha_proteins_crosslink_positions"] == [145]
    assert csm["alpha_proteins_peptide_positions"] == [145]
    assert csm["alpha_score"] == pytest.approx(77.979659)
    assert not csm["alpha_decoy"]
    assert csm["beta_peptide"] == "KLVDSTDK"
    assert csm["beta_modifications"] is None
    assert csm["beta_peptide_crosslink_position"] == 1
    assert csm["beta_proteins"] == ["Cas9"]
    assert csm["beta_proteins_crosslink_positions"] == [145]
    assert csm["beta_proteins_peptide_positions"] == [145]
    assert csm["beta_score"] == pytest.approx(77.979659)
    assert not csm["beta_decoy"]
    assert csm["crosslink_type"] == "intra"
    assert csm["score"] == pytest.approx(139)
    assert csm["spectrum_file"] == "XLpeplib_Beveridge_QEx-HFX_DSS_R1"
    assert csm["scan_nr"] == 8258
    assert csm["charge"] == 4
    assert csm["retention_time"] == pytest.approx(2605.0)
    assert csm["ion_mobility"] is None



#!/usr/bin/env python3

# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

from __future__ import annotations

import warnings
from matplotlib import pyplot as plt
from matplotlib.figure import Figure
from matplotlib_venn import venn2, venn2_circles
from matplotlib_venn import venn3, venn3_circles

from ..data import check_input
from ..transform.util import get_available_keys

from typing import Optional
from typing import List
from typing import Dict
from typing import Tuple
from typing import Set
from typing import Any

# legacy
try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


def __get_key(data: Dict[str, Any], by: Literal["peptide", "protein"]) -> str:
    r"""Get the unique key for a crosslink-spectrum-match or crosslink.

    Parameters
    ----------
    data : dict of str, any
        A pyXLMS crosslink-spectrum-match or crosslink object.
    by : str, one of "peptide" or "protein"
        If peptide or protein crosslink position should be used for determining if a crosslink-spectrum-match
        or crosslink is unique.

    Returns
    -------
    str
        The unique key for the crosslink-spectrum-match or crosslink.

    Notes
    -----
    This function should not be called directly, it is called from ``plot_venn_diagram()``.
    """
    if by == "peptide":
        return f"{data['alpha_peptide']}_{data['alpha_peptide_crosslink_position']}-{data['beta_peptide']}_{data['beta_peptide_crosslink_position']}"
    prot_pos_a = "-".join(
        sorted(
            [
                f"{data['alpha_proteins'][i]}_{data['alpha_proteins_crosslink_positions'][i]}"
                for i in range(len(data["alpha_proteins"]))
            ]
        )
    )
    prot_pos_b = "-".join(
        sorted(
            [
                f"{data['beta_proteins'][i]}_{data['beta_proteins_crosslink_positions'][i]}"
                for i in range(len(data["beta_proteins"]))
            ]
        )
    )
    return ":".join(sorted([prot_pos_a, prot_pos_b]))


def venn(
    set_1: Set[Any],
    set_2: Set[Any],
    set_3: Optional[Set[Any]] = None,
    labels: List[str] = ["Set 1", "Set 2", "Set 3"],
    colors: List[str] = ["#4361EE", "#4CC9F0", "#F72585"],
    alpha: float = 0.6,
    contour: bool = False,
    linewidth: float = 0.5,
    title: str = "Venn Diagram",
    figsize: Tuple[float, float] = (10.0, 10.0),
    filename_prefix: Optional[str] = None,
) -> Tuple[Figure, Any]:
    r"""Wrapper with pre-set defaults for creating venn diagrams with the matplotlib-venn package.

    Wrapper with pre-set defaults for creating venn diagrams with the matplotlib-venn package
    `github.com/konstantint/matplotlib-venn <https://github.com/konstantint/matplotlib-venn>`_.

    Parameters
    ----------
    set_1 : set
        First set of the venn diagram.
    set_2 : set
        Second set of the venn diagram.
    set_3 : set, or None, default = None
        If not None a three set venn diagram will be drawn, if None
        the two set venn diagram of ``set_1`` and ``set_2`` will be drawn.
    labels : List[str], default = ["Set 1", "Set 2", "Set 3"]
        List of labels for the sets.
    colors : List[str], default = ["#4361EE", "#4CC9F0", "#F72585"]
        List of valid colors to use for the venn circles.
    alpha : float, default = 0.6
        Color opacity.
    contour : bool, default = False
        If a contour should be drawn around venn circles.
    linewidth : float, default = 0.5
        Linewidth of the contour.
    title : str, default = "Venn Diagram"
        Title of the venn diagram.
    figsize : tuple of float, float, default = (10.0, 10.0)
        Width, height in inches.
    filename_prefix: str, or None, default = None
        If given, plot will be saved with and without title in .png and .svg format with the given
        prefix.

    Returns
    -------
    tuple of matplotlib.figure.Figure, any
        The created figure and axis ``from matplotlib.pyplot.subplots()``.

    Warns
    -----
    RuntimeWarning
        If more labels or colors than sets are supplied.

    Raises
    ------
    IndexError
        If less labels or colors than sets are supplied.

    Examples
    --------
    >>> from pyXLMS.plotting import venn
    >>> fig, ax = venn(
    ...     {"A", "B", "C"},
    ...     {"B", "C", "D", "E", "F"},
    ...     labels=["A", "F"],
    ...     colors=["orange", "blue"],
    ... )

    >>> from pyXLMS.plotting import venn
    >>> fig, ax = venn({"A", "B", "C"}, {"B", "C", "D", "E", "F"}, {"F", "G"})
    """

    fig, ax = plt.subplots(figsize=figsize)

    if set_3 is None:
        # checks
        if len(labels) > 2:
            warnings.warn(
                RuntimeWarning(
                    "More than two labels supplied for two sets. Using first two..."
                )
            )
            labels = labels[:2]

        if len(labels) < 2:
            raise IndexError(
                "At least two labels have to be given if two sets are supplied!"
            )

        if len(colors) > 2:
            warnings.warn(
                RuntimeWarning(
                    "More than two colors supplied for two sets. Using first two..."
                )
            )
            colors = colors[:2]

        if len(colors) < 2:
            raise IndexError(
                "At least two colors have to be given if two sets are supplied!"
            )

        # create 2 set venn diagram
        venn2(
            subsets=(
                len(set_1.difference(set_2)),
                len(set_2.difference(set_1)),
                len(set_1.intersection(set_2)),
            ),
            set_labels=tuple(labels),  # pyright: ignore[reportArgumentType]
            set_colors=tuple(colors),  # pyright: ignore[reportArgumentType]
            alpha=alpha,
        )

        # draw contour
        if contour:
            venn2_circles(
                subsets=(
                    len(set_1.difference(set_2)),
                    len(set_2.difference(set_1)),
                    len(set_1.intersection(set_2)),
                ),
                linewidth=linewidth,
            )

        # save file
        if filename_prefix is not None:
            plt.savefig(
                filename_prefix + "_notitle.png",
                dpi=300,
                transparent=True,
                bbox_inches="tight",
            )
            plt.savefig(
                filename_prefix + "_notitle.svg",
                dpi=300,
                transparent=True,
                bbox_inches="tight",
            )
            plt.title(title)
            plt.savefig(
                filename_prefix + ".png", dpi=300, transparent=True, bbox_inches="tight"
            )
            plt.savefig(
                filename_prefix + ".svg", dpi=300, transparent=True, bbox_inches="tight"
            )
        else:
            plt.title(title)

    else:
        # checks
        if len(labels) > 3:
            warnings.warn(
                RuntimeWarning(
                    "More than three labels supplied for three sets. Using first three..."
                )
            )
            labels = labels[:3]

        if len(labels) < 3:
            raise IndexError(
                "At least three labels have to be given if three sets are supplied!"
            )

        if len(colors) > 3:
            warnings.warn(
                RuntimeWarning(
                    "More than three colors supplied for three sets. Using first three..."
                )
            )
            colors = colors[:3]

        if len(colors) < 3:
            raise IndexError(
                "At least three colors have to be given if three sets are supplied!"
            )

        # create 3 set venn diagram
        venn3(
            subsets=(
                len(set_1.difference(set_2, set_3)),
                len(set_2.difference(set_1, set_3)),
                len(set_1.intersection(set_2).difference(set_3)),
                len(set_3.difference(set_1, set_2)),
                len(set_1.intersection(set_3).difference(set_2)),
                len(set_2.intersection(set_3).difference(set_1)),
                len(set_1.intersection(set_3).intersection(set_2)),
            ),
            set_labels=tuple(labels),  # pyright: ignore[reportArgumentType]
            set_colors=tuple(colors),  # pyright: ignore[reportArgumentType]
            alpha=alpha,
        )

        # draw contour
        if contour:
            venn3_circles(
                subsets=(
                    len(set_1.difference(set_2, set_3)),
                    len(set_2.difference(set_1, set_3)),
                    len(set_1.intersection(set_2).difference(set_3)),
                    len(set_3.difference(set_1, set_2)),
                    len(set_1.intersection(set_3).difference(set_2)),
                    len(set_2.intersection(set_3).difference(set_1)),
                    len(set_1.intersection(set_3).intersection(set_2)),
                ),
                linewidth=linewidth,  # pyright: ignore[reportArgumentType]
            )

        # save file
        if filename_prefix is not None:
            plt.savefig(
                filename_prefix + "_notitle.png",
                dpi=300,
                transparent=True,
                bbox_inches="tight",
            )
            plt.savefig(
                filename_prefix + "_notitle.svg",
                dpi=300,
                transparent=True,
                bbox_inches="tight",
            )
            plt.title(title)
            plt.savefig(
                filename_prefix + ".png", dpi=300, transparent=True, bbox_inches="tight"
            )
            plt.savefig(
                filename_prefix + ".svg", dpi=300, transparent=True, bbox_inches="tight"
            )
        else:
            plt.title(title)

    return (fig, ax)


def plot_venn_diagram(
    data_1: List[Dict[str, Any]],
    data_2: List[Dict[str, Any]],
    data_3: Optional[List[Dict[str, Any]]] = None,
    by: Literal["peptide", "protein"] = "peptide",
    labels: List[str] = ["Set 1", "Set 2", "Set 3"],
    colors: List[str] = ["#4361EE", "#4CC9F0", "#F72585"],
    alpha: float = 0.6,
    contour: bool = False,
    linewidth: float = 0.5,
    title: str = "Venn Diagram",
    figsize: Tuple[float, float] = (10.0, 10.0),
    filename_prefix: Optional[str] = None,
) -> Tuple[Figure, Any]:
    r"""Plot the venn diagram for two or three sets of crosslink-spectrum-matches or crosslinks.

    Plot the venn diagram for two or three sets of crosslink-spectrum-matches or crosslinks. Overlaps
    are calculated by either looking at peptide sequence and crosslink position in the peptide using
    parameter by = "peptide" or by looking at protein crosslink position by using parameter by = "protein".
    Please note that crosslink-spectrum-matches are automatically aggregated to crosslinks, and scan numbers
    do not influence the creation of the venn diagram. For more nuanced control over intersecting
    crosslink-spectrum-matches with scan numbers please refer to ``transform.intersection()``.

    Parameters
    ----------
    data_1 : list of dict of str, any
        A list of crosslink-spectrum-matches or crosslinks.
    data_2 : list of dict of str, any
        A list of crosslink-spectrum-matches or crosslinks.
    data_3 : list of dict of str, any, or None, default = None
        Optionally, a third list of crosslink-spectrum-matches or crosslinks.
    by : str, one of "peptide" or "protein"
        If peptide or protein crosslink position should be used for determining if a crosslink-spectrum-match
        or crosslink is unique.
    labels : List[str], default = ["Set 1", "Set 2", "Set 3"]
        List of labels for the sets.
    colors : List[str], default = ["#4361EE", "#4CC9F0", "#F72585"]
        List of valid colors to use for the venn circles.
    alpha : float, default = 0.6
        Color opacity.
    contour : bool, default = False
        If a contour should be drawn around venn circles.
    linewidth : float, default = 0.5
        Linewidth of the contour.
    title : str, default = "Venn Diagram"
        Title of the venn diagram.
    figsize : tuple of float, float, default = (10.0, 10.0)
        Width, height in inches.
    filename_prefix: str, or None, default = None
        If given, plot will be saved with and without title in .png and .svg format with the given
        prefix.

    Returns
    -------
    tuple of matplotlib.figure.Figure, any
        The created figure and axis ``from matplotlib.pyplot.subplots()``.

    Raises
    ------
    TypeError
        If a wrong data type is provided.
    TypeError
        If parameter by is not one of 'peptide' or 'protein'.
    ValueError
        If one of the data parameters does not contain any crosslink-spectrum-matches or crosslinks.
    ValueError
        If attribute 'alpha_proteins', 'alpha_proteins_crosslink_positions', 'beta_proteins', or
        'beta_proteins_crosslink_positions' is not available for any of the data and parameter 'by'
        was set to 'protein'.

    Notes
    -----
    Please note that crosslink-spectrum-matches are automatically aggregated to crosslinks, and scan numbers
    do not influence the creation of the venn diagram. For more nuanced control over intersecting
    crosslink-spectrum-matches with scan numbers please refer to ``transform.intersection()``.

    Examples
    --------
    >>> from pyXLMS import parser
    >>> from pyXLMS import plotting
    >>> a = parser.read(
    ...     "data/ms_annika/XLpeplib_Beveridge_QEx-HFX_DSS_R1_CSMs.txt",
    ...     engine="MS Annika",
    ...     crosslinker="DSS",
    ... )
    >>> a = a["crosslink-spectrum-matches"]
    >>> b = parser.read(
    ...     "data/maxquant/run1/crosslinkMsms.txt", engine="MaxQuant", crosslinker="DSS"
    ... )
    >>> b = b["crosslink-spectrum-matches"]
    >>> fig, ax = plotting.plot_venn_diagram(
    ...     a, b, labels=["MS Annika", "MaxQuant"], colors=["orange", "blue"]
    ... )

    >>> from pyXLMS import parser
    >>> from pyXLMS import plotting
    >>> a = parser.read(
    ...     "data/ms_annika/XLpeplib_Beveridge_QEx-HFX_DSS_R1_CSMs.txt",
    ...     engine="MS Annika",
    ...     crosslinker="DSS",
    ... )
    >>> a = a["crosslink-spectrum-matches"]
    >>> b = parser.read(
    ...     "data/maxquant/run1/crosslinkMsms.txt", engine="MaxQuant", crosslinker="DSS"
    ... )
    >>> b = b["crosslink-spectrum-matches"]
    >>> c = parser.read(
    ...     "data/plink2/Cas9_plus10_2024.06.20.filtered_cross-linked_spectra.csv",
    ...     engine="pLink",
    ...     crosslinker="DSS",
    ... )
    >>> c = c["crosslink-spectrum-matches"]
    >>> fig, ax = plotting.plot_venn_diagram(
    ...     a, b, c, labels=["MS Annika", "MaxQuant", "pLink"], contour=True
    ... )
    """
    _ok = check_input(data_1, "data_1", list, dict)
    _ok = check_input(data_2, "data_2", list, dict)
    _ok = check_input(data_3, "data_3", list, dict) if data_3 is not None else True
    _ok = check_input(by, "by", str)
    _ok = check_input(labels, "labels", list, str)
    _ok = check_input(colors, "colors", list, str)
    _ok = check_input(alpha, "alpha", float)
    _ok = check_input(contour, "contour", bool)
    _ok = check_input(linewidth, "linewidth", float)
    _ok = check_input(title, "title", str)
    _ok = check_input(figsize, "figsize", tuple)
    _ok = (
        check_input(filename_prefix, "filename_prefix", str)
        if filename_prefix is not None
        else True
    )
    if by not in ["peptide", "protein"]:
        raise TypeError(
            "Parameter 'by' has to be one of 'peptide' or 'protein'! Option 'peptide' will group by peptide sequence and "
            "peptide crosslink position while option 'protein' will group by protein identifier and protein crosslink position."
        )
    if len(data_1) == 0:
        raise ValueError(
            "Can't plot venn diagram if no crosslink-spectrum-matches or crosslinks are given in data_1!"
        )
    if len(data_2) == 0:
        raise ValueError(
            "Can't plot venn diagram if no crosslink-spectrum-matches or crosslinks are given in data_2!"
        )
    if data_3 is not None and len(data_3) == 0:
        raise ValueError(
            "Can't plot 3-set venn diagram if no crosslink-spectrum-matches or crosslinks are given in data_3!"
        )
    if "data_type" not in data_1[0] or data_1[0]["data_type"] not in [
        "crosslink",
        "crosslink-spectrum-match",
    ]:
        raise TypeError(
            "Unsupported data type for input data! Parameter data_1 has to be a list of crosslink or crosslink-spectrum-match!"
        )
    if "data_type" not in data_2[0] or data_2[0]["data_type"] not in [
        "crosslink",
        "crosslink-spectrum-match",
    ]:
        raise TypeError(
            "Unsupported data type for input data! Parameter data_2 has to be a list of crosslink or crosslink-spectrum-match!"
        )
    if data_3 is not None:
        if "data_type" not in data_3[0] or data_3[0]["data_type"] not in [
            "crosslink",
            "crosslink-spectrum-match",
        ]:
            raise TypeError(
                "Unsupported data type for input data! Parameter data_3 has to be a list of crosslink or crosslink-spectrum-match, or None!"
            )

    set_1 = set()
    set_2 = set()
    set_3 = set()
    if by == "protein":
        available_keys_1 = get_available_keys(data_1)
        available_keys_2 = get_available_keys(data_2)
        if data_3 is None:
            alpha_proteins = (
                available_keys_1["alpha_proteins"]
                and available_keys_2["alpha_proteins"]
            )
            alpha_proteins_crosslink_positions = (
                available_keys_1["alpha_proteins_crosslink_positions"]
                and available_keys_2["alpha_proteins_crosslink_positions"]
            )
            beta_proteins = (
                available_keys_1["beta_proteins"] and available_keys_2["beta_proteins"]
            )
            beta_proteins_crosslink_positions = (
                available_keys_1["beta_proteins_crosslink_positions"]
                and available_keys_2["beta_proteins_crosslink_positions"]
            )
            if (
                not alpha_proteins
                or not alpha_proteins_crosslink_positions
                or not beta_proteins
                or not beta_proteins_crosslink_positions
            ):
                raise ValueError(
                    "Grouping by protein crosslink position is only available if all data have defined protein crosslink positions!\n"
                    "This error might be fixable with 'transform.reannotate_positions()'!"
                )
            for item in data_1:
                set_1.add(__get_key(item, by))
            for item in data_2:
                set_2.add(__get_key(item, by))
        else:
            available_keys_3 = get_available_keys(data_3)
            alpha_proteins = (
                available_keys_1["alpha_proteins"]
                and available_keys_2["alpha_proteins"]
                and available_keys_3["alpha_proteins"]
            )
            alpha_proteins_crosslink_positions = (
                available_keys_1["alpha_proteins_crosslink_positions"]
                and available_keys_2["alpha_proteins_crosslink_positions"]
                and available_keys_3["alpha_proteins_crosslink_positions"]
            )
            beta_proteins = (
                available_keys_1["beta_proteins"]
                and available_keys_2["beta_proteins"]
                and available_keys_3["beta_proteins"]
            )
            beta_proteins_crosslink_positions = (
                available_keys_1["beta_proteins_crosslink_positions"]
                and available_keys_2["beta_proteins_crosslink_positions"]
                and available_keys_3["beta_proteins_crosslink_positions"]
            )
            if (
                not alpha_proteins
                or not alpha_proteins_crosslink_positions
                or not beta_proteins
                or not beta_proteins_crosslink_positions
            ):
                raise ValueError(
                    "Grouping by protein crosslink position is only available if all data have defined protein crosslink positions!\n"
                    "This error might be fixable with 'transform.reannotate_positions()'!"
                )
            for item in data_1:
                set_1.add(__get_key(item, by))
            for item in data_2:
                set_2.add(__get_key(item, by))
            for item in data_3:
                set_3.add(__get_key(item, by))
    else:
        if data_3 is None:
            for item in data_1:
                set_1.add(__get_key(item, by))
            for item in data_2:
                set_2.add(__get_key(item, by))
        else:
            for item in data_1:
                set_1.add(__get_key(item, by))
            for item in data_2:
                set_2.add(__get_key(item, by))
            for item in data_3:
                set_3.add(__get_key(item, by))

    return venn(
        set_1,
        set_2,
        set_3 if data_3 is not None else None,
        labels=labels,
        colors=colors,
        alpha=alpha,
        contour=contour,
        linewidth=linewidth,
        title=title,
        figsize=figsize,
        filename_prefix=filename_prefix,
    )



#!/usr/bin/env python3

# pyXLMS - TESTS
# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com


def test1():
    import pyXLMS

    pr = pyXLMS.parser.read(
        "data/ms_annika/XLpeplib_Beveridge_QEx-HFX_DSS_R1_CSMs.xlsx",
        engine="MS Annika",
        crosslinker="DSS",
    )
    _ = pyXLMS.transform.summary(pr)
    _ = pyXLMS.exporter.to_xifdr(
        pr["crosslink-spectrum-matches"], filename="msannika_CSMs_for_xiFDR.csv"
    )
    assert pr is not None



#!/usr/bin/env python3

# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

from __future__ import annotations

from matplotlib import pyplot as plt
from matplotlib.figure import Figure

from ..data import check_input
from ..transform.util import get_available_keys
from ..transform.filter import filter_target_decoy

from typing import Optional
from typing import List
from typing import Dict
from typing import Tuple
from typing import Any


def plot_target_decoy_distribution(
    data: List[Dict[str, Any]],
    colors: List[str] = ["#00a087", "#3c5488", "#e64b35"],
    title: str = "Target and Decoy Distribution",
    figsize: Tuple[float, float] = (16.0, 9.0),
    filename_prefix: Optional[str] = None,
) -> Tuple[Figure, Any]:
    r"""Plot the target-decoy distribution for a set of crosslink-spectrum-matches or crosslinks.

    Plot the target-target, target-decoy, and decoy-decoy distribution as a barplot for a
    set of crosslink-spectrum-matches or crosslinks.

    Parameters
    ----------
    data : list of dict of str, any
        A list of crosslink-spectrum-matches or crosslinks.
    colors : list of str, default = ["#00a087", "#3c5488", "#e64b35"]
        Colors of the bars.
    title : str, default = "Target and Decoy Distribution"
        The title of the barplot.
    figsize : tuple of float, float, default = (16.0, 9.0)
        Width, height in inches.
    filename_prefix : str, or None
        If given, plot will be saved with and without title in .png and .svg format with the given
        prefix.

    Returns
    -------
    tuple of matplotlib.figure.Figure, any
        The created figure and axis ``from matplotlib.pyplot.subplots()``.

    Raises
    ------
    TypeError
        If a wrong data type is provided.
    ValueError
        If parameter data does not contain any crosslink-spectrum-matches or crosslinks.
    ValueError
        If attribute 'alpha_decoy', or 'beta_decoy' is not available for any of the data.

    Examples
    --------
    >>> from pyXLMS import parser
    >>> from pyXLMS import plotting
    >>> pr = parser.read_msannika(
    ...     "data/ms_annika/XLpeplib_Beveridge_QEx-HFX_DSS_R1_CSMs.xlsx"
    ... )
    >>> csms = pr["crosslink-spectrum-matches"]
    >>> fig, ax = plotting.plot_target_decoy_distribution(csms)
    """
    _ok = check_input(data, "data", list, dict)
    _ok = check_input(colors, "colors", list, str)
    _ok = check_input(title, "title", str)
    _ok = check_input(figsize, "figsize", tuple)
    _ok = (
        check_input(filename_prefix, "filename_prefix", str)
        if filename_prefix is not None
        else True
    )
    if len(data) == 0:
        raise ValueError(
            "Can't plot target-decoy distribution if no crosslink-spectrum-matches or crosslinks are given!"
        )
    if "data_type" not in data[0] or data[0]["data_type"] not in [
        "crosslink",
        "crosslink-spectrum-match",
    ]:
        raise TypeError(
            "Unsupported data type for input data! Parameter data has to be a list of crosslink or crosslink-spectrum-match!"
        )
    available_keys = get_available_keys(data)
    if not available_keys["alpha_decoy"] or not available_keys["beta_decoy"]:
        raise ValueError(
            "Can't plot target-decoy distribution if target/decoy labels are missing!"
        )
    ylabel = (
        "crosslink-spectrum-matches"
        if data[0]["data_type"] == "crosslink-spectrum-match"
        else "crosslinks"
    )
    filtered = filter_target_decoy(data)
    tt = len(filtered["Target-Target"])
    td = len(filtered["Target-Decoy"])
    dd = len(filtered["Decoy-Decoy"])

    fig, ax = plt.subplots(figsize=figsize)

    bar = ax.bar(
        ["Target-Target", "Target-Decoy", "Decoy-Decoy"],
        [tt, td, dd],
        color=colors,
    )
    ax.bar_label(bar, padding=3.0)
    ax.set_ylabel(f"Number of {ylabel}")
    ax.set_xlabel("Type")

    if filename_prefix is not None:
        plt.savefig(
            filename_prefix + "_notitle.png",
            dpi=300,
            transparent=True,
            bbox_inches="tight",
        )
        plt.savefig(
            filename_prefix + "_notitle.svg",
            dpi=300,
            transparent=True,
            bbox_inches="tight",
        )
        ax.set_title(title)
        plt.savefig(
            filename_prefix + ".png", dpi=300, transparent=True, bbox_inches="tight"
        )
        plt.savefig(
            filename_prefix + ".svg", dpi=300, transparent=True, bbox_inches="tight"
        )
    else:
        ax.set_title(title)

    return (fig, ax)



#!/usr/bin/env python3

# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

from __future__ import annotations

import pandas as pd
from matplotlib import pyplot as plt
from matplotlib.figure import Figure

from ..data import check_input
from ..transform.util import get_available_keys
from ..transform.filter import filter_protein_distribution
from ..transform.filter import filter_crosslink_type

from typing import Optional
from typing import List
from typing import Dict
from typing import Tuple
from typing import Any


def plot_protein_distribution(
    data: List[Dict[str, Any]],
    top_n: int = 25,
    colors: List[str] = ["#6d4bff", "#ac99ff"],
    title: str = "Protein Distribution",
    figsize: Tuple[float, float] = (16.0, 9.0),
    filename_prefix: Optional[str] = None,
) -> Tuple[Figure, Any]:
    r"""Plot the protein distribution for a set of crosslink-spectrum-matches or crosslinks.

    Plot the protein distribution as a barplot for a set of crosslink-spectrum-matches or crosslinks.

    Parameters
    ----------
    data : list of dict of str, any
        A list of crosslink-spectrum-matches or crosslinks.
    top_n : int, default = 25
        Number of proteins to plot. Proteins are sorted by number of crosslinks
        or crosslink-spectrum-matches.
    colors : list of str, default = ["#6d4bff", "#ac99ff"]
        Colors of the bar-types (intra-link and inter-link).
    title : str, default = "Protein Distribution"
        The title of the barplot.
    figsize : tuple of float, float, default = (16.0, 9.0)
        Width, height in inches.
    filename_prefix : str, or None
        If given, plot will be saved with and without title in .png and .svg format with the given
        prefix.

    Returns
    -------
    tuple of matplotlib.figure.Figure, any
        The created figure and axis ``from matplotlib.pyplot.subplots()``.

    Raises
    ------
    TypeError
        If a wrong data type is provided.
    ValueError
        If parameter data does not contain any crosslink-spectrum-matches or crosslinks.
    ValueError
        If attribute 'alpha_proteins', or 'beta_proteins' is not available for any of the data.
    IndexError
        If not enough colors where specified.

    Examples
    --------
    >>> from pyXLMS import parser
    >>> from pyXLMS import plotting
    >>> pr = parser.read_msannika(
    ...     "data/ms_annika/XLpeplib_Beveridge_QEx-HFX_DSS_R1_CSMs.xlsx"
    ... )
    >>> csms = pr["crosslink-spectrum-matches"]
    >>> fig, ax = plotting.plot_protein_distribution(csms)
    """
    _ok = check_input(data, "data", list, dict)
    _ok = check_input(top_n, "top_n", int)
    _ok = check_input(colors, "colors", list, str)
    _ok = check_input(title, "title", str)
    _ok = check_input(figsize, "figsize", tuple)
    _ok = (
        check_input(filename_prefix, "filename_prefix", str)
        if filename_prefix is not None
        else True
    )
    if len(colors) < 2:
        raise IndexError("At least two colors need to be given for the plot!")
    if len(data) == 0:
        raise ValueError(
            "Can't plot protein distribution if no crosslink-spectrum-matches or crosslinks are given!"
        )
    if "data_type" not in data[0] or data[0]["data_type"] not in [
        "crosslink",
        "crosslink-spectrum-match",
    ]:
        raise TypeError(
            "Unsupported data type for input data! Parameter data has to be a list of crosslink or crosslink-spectrum-match!"
        )
    available_keys = get_available_keys(data)
    if not available_keys["alpha_proteins"] or not available_keys["beta_proteins"]:
        raise ValueError(
            "Can't plot protein distribution if attributes 'alpha_proteins' or 'beta_proteins' are missing!"
        )
    ylabel = (
        "crosslink-spectrum-matches"
        if data[0]["data_type"] == "crosslink-spectrum-match"
        else "crosslinks"
    )
    proteins = filter_protein_distribution(data)
    protein_names = list()
    protein_intra = list()
    protein_inter = list()
    protein_total = list()
    for protein in proteins:
        protein_names.append(protein)
        intra_inter = filter_crosslink_type(proteins[protein])
        protein_intra.append(len(intra_inter["Intra"]))
        protein_inter.append(len(intra_inter["Inter"]))
        protein_total.append(len(proteins[protein]))

    sorted = pd.DataFrame(
        {
            "protein": protein_names,
            "intra": protein_intra,
            "inter": protein_inter,
            "total": protein_total,
        }
    ).sort_values(by="total", axis=0, ascending=False)
    protein_names = sorted["protein"].values.tolist()[:top_n]
    protein_intra = sorted["intra"].values.tolist()[:top_n]
    protein_inter = sorted["inter"].values.tolist()[:top_n]

    fig, ax = plt.subplots(figsize=figsize)
    bottom = [0.0 for i in protein_names]

    bar = ax.bar(
        protein_names, protein_intra, label="intra-link", bottom=bottom, color=colors[0]
    )
    bottom = protein_intra
    ax.bar_label(bar, label_type="center")

    bar = ax.bar(
        protein_names, protein_inter, label="inter-link", bottom=bottom, color=colors[1]
    )
    bottom = protein_inter
    ax.bar_label(bar, label_type="center")

    ax.set_xticks(range(len(protein_names)), protein_names, rotation=45, ha="right")
    ax.legend(loc="upper right")
    ax.set_ylabel(f"Number of {ylabel}")
    ax.set_xlabel("Protein")

    if filename_prefix is not None:
        plt.savefig(
            filename_prefix + "_notitle.png",
            dpi=300,
            transparent=True,
            bbox_inches="tight",
        )
        plt.savefig(
            filename_prefix + "_notitle.svg",
            dpi=300,
            transparent=True,
            bbox_inches="tight",
        )
        ax.set_title(title)
        plt.savefig(
            filename_prefix + ".png", dpi=300, transparent=True, bbox_inches="tight"
        )
        plt.savefig(
            filename_prefix + ".svg", dpi=300, transparent=True, bbox_inches="tight"
        )
    else:
        ax.set_title(title)

    return (fig, ax)



#!/usr/bin/env python3

# 2024 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

from __future__ import annotations

from ..data import check_input

from typing import Optional
from typing import Dict
from typing import Tuple
from typing import List
from typing import Any


def modifications_to_str(
    modifications: Optional[Dict[int, Tuple[str, float]]],
) -> str | None:
    r"""Returns the string representation of a modifications dictionary.

    Parameters
    ----------
    modifications : dict of [str, tuple], or None
        The modifications of a peptide given as a dictionary that maps peptide position (1-based) to modification given as a tuple of modification name and modification delta mass.
        ``N-terminal`` modifications should be denoted with position ``0``. ``C-terminal`` modifications should be denoted with position ``len(peptide) + 1``.

    Returns
    -------
    str, or None
        The string representation of the modifications (or ``None`` if no modification was provided).

    Examples
    --------
    >>> from pyXLMS.transform import modifications_to_str
    >>> modifications_to_str(
    ...     {1: ("Oxidation", 15.994915), 5: ("Carbamidomethyl", 57.021464)}
    ... )
    '(1:[Oxidation|15.994915]);(5:[Carbamidomethyl|57.021464])'
    """
    ## check input
    _ok = (
        check_input(modifications, "modifcations", dict, tuple)
        if modifications is not None
        else True
    )

    modifications_str = ""
    if modifications is None:
        return None
    for modification_pos in sorted(modifications.keys()):
        modifications_str += f"({modification_pos}:[{modifications[modification_pos][0]}|{modifications[modification_pos][1]}]);"
    return modifications_str.rstrip(";")


def assert_data_type_same(data_list: List[Dict[str, Any]]) -> bool:
    r"""Checks that all data is of the same data type.

    Verifies that all elements in the provided list are of the same data type.

    Parameters
    ----------
    data_list : list of dict of str, any
        A list of dictionaries with the ``data_type`` key.

    Returns
    -------
    bool
        If all elements are of the same data type.

    Examples
    --------
    >>> from pyXLMS.transform import assert_data_type_same
    >>> from pyXLMS import data
    >>> data_list = [
    ...     data.create_crosslink_min("PEPK", 4, "PKEP", 2),
    ...     data.create_crosslink_min("KPEP", 1, "PEKP", 3),
    ... ]
    >>> assert_data_type_same(data_list)
    True

    >>> from pyXLMS.transform import assert_data_type_same
    >>> from pyXLMS import data
    >>> data_list = [
    ...     data.create_crosslink_min("PEPK", 4, "PKEP", 2),
    ...     data.create_csm_min("KPEP", 1, "PEKP", 3, "RUN_1", 1),
    ... ]
    >>> assert_data_type_same(data_list)
    False
    """
    _ok = check_input(data_list, "data_list", list, dict)
    data_type = data_list[0]["data_type"]
    for item in data_list[1:]:
        if item["data_type"] != data_type:
            return False
    return True


def get_available_keys(data_list: List[Dict[str, Any]]) -> Dict[str, bool]:
    r"""Checks which data is available from a list of crosslinks or crosslink-spectrum-matches.

    Verifies which data fields have been set for all crosslinks or crosslink-spectrum-matches in the
    given list. Will return a dictionary structured the same as a crosslink or crosslink-spectrum-match,
    but instead of the data it will return either True or False, depending if the field was set or not.

    Parameters
    ----------
    data_list : list of dict of str, any
        A list of crosslinks or crosslink-spectrum-matches.

    Returns
    -------
    dict of str, bool
        - If a list of crosslinks was provided, a dictionary with the following keys will be returned, where the value
          of each key denotes if the data field is available for all crosslinks in ``data_list``.
          Keys: ``data_type``, ``completeness``, ``alpha_peptide``, ``alpha_peptide_crosslink_position``,
          ``alpha_proteins``, ``alpha_proteins_crosslink_positions``, ``alpha_decoy``, ``beta_peptide``, ``beta_peptide_crosslink_position``,
          ``beta_proteins``, ``beta_proteins_crosslink_positions``, ``beta_decoy``, ``crosslink_type``, ``score``, and ``additional_information``.
        - If a list of crosslink-spectrum-matches was provided, a dictionary with the following keys will be returned, where the value
          of each key denotes if the data field is available for all crosslink-spectrum-matches in ``data_list``.
          Keys: ``data_type``, ``completeness``, ``alpha_peptide``, ``alpha_modifications``,
          ``alpha_peptide_crosslink_position``, ``alpha_proteins``, ``alpha_proteins_crosslink_positions``, ``alpha_proteins_peptide_positions``,
          ``alpha_score``, ``alpha_decoy``, ``beta_peptide``, ``beta_modifications``, ``beta_peptide_crosslink_position``, ``beta_proteins``,
          ``beta_proteins_crosslink_positions``, ``beta_proteins_peptide_positions``, ``beta_score``, ``beta_decoy``, ``crosslink_type``, ``score``,
          ``spectrum_file``, ``scan_nr``, ``retention_time``, ``ion_mobility``, and ``additional_information``.

    Raises
    ------
    TypeError
        If not all elements in ``data_list`` are of the same data type.
    TypeError
        If one or more elements in the list are of an unsupported data type.

    Examples
    --------
    >>> from pyXLMS.transform import get_available_keys
    >>> from pyXLMS import data
    >>> data_list = [
    ...     data.create_crosslink_min("PEPK", 4, "PKEP", 2),
    ...     data.create_crosslink_min("KPEP", 1, "PEKP", 3),
    ... ]
    >>> available_keys = get_available_keys(data_list)
    >>> available_keys["alpha_peptide"]
    True
    >>> available_keys["score"]
    False
    """
    if not assert_data_type_same(data_list):
        raise TypeError("Not all elements of the list have the same data type!")
    data_type = data_list[0]["data_type"]
    # available keys
    modifications_a = True
    proteins_a = True
    xl_position_proteins_a = True
    pep_position_proteins_a = True
    score_a = True
    decoy_a = True
    modifications_b = True
    proteins_b = True
    xl_position_proteins_b = True
    pep_position_proteins_b = True
    score_b = True
    decoy_b = True
    score = True
    charge = True
    rt = True
    im_cv = True
    additional_information = True
    # parse available keys
    if data_type == "crosslink":
        for data in data_list:
            if data["completeness"] != "full":
                if data["alpha_proteins"] is None:
                    proteins_a = False
                if data["alpha_proteins_crosslink_positions"] is None:
                    xl_position_proteins_a = False
                if data["alpha_decoy"] is None:
                    decoy_a = False
                if data["beta_proteins"] is None:
                    proteins_b = False
                if data["beta_proteins_crosslink_positions"] is None:
                    xl_position_proteins_b = False
                if data["beta_decoy"] is None:
                    decoy_b = False
                if data["score"] is None:
                    score = False
                if data["additional_information"] is None:
                    additional_information = False
        return {
            "data_type": True,
            "completeness": True,
            "alpha_peptide": True,
            "alpha_peptide_crosslink_position": True,
            "alpha_proteins": proteins_a,
            "alpha_proteins_crosslink_positions": xl_position_proteins_a,
            "alpha_decoy": decoy_a,
            "beta_peptide": True,
            "beta_peptide_crosslink_position": True,
            "beta_proteins": proteins_b,
            "beta_proteins_crosslink_positions": xl_position_proteins_b,
            "beta_decoy": decoy_b,
            "crosslink_type": True,
            "score": score,
            "additional_information": additional_information,
        }
    if data_type == "crosslink-spectrum-match":
        for data in data_list:
            if data["completeness"] != "full":
                if data["alpha_modifications"] is None:
                    modifications_a = False
                if data["alpha_proteins"] is None:
                    proteins_a = False
                if data["alpha_proteins_crosslink_positions"] is None:
                    xl_position_proteins_a = False
                if data["alpha_proteins_peptide_positions"] is None:
                    pep_position_proteins_a = False
                if data["alpha_score"] is None:
                    score_a = False
                if data["alpha_decoy"] is None:
                    decoy_a = False
                if data["beta_modifications"] is None:
                    modifications_b = False
                if data["beta_proteins"] is None:
                    proteins_b = False
                if data["beta_proteins_crosslink_positions"] is None:
                    xl_position_proteins_b = False
                if data["beta_proteins_peptide_positions"] is None:
                    pep_position_proteins_b = False
                if data["beta_score"] is None:
                    score_b = False
                if data["beta_decoy"] is None:
                    decoy_b = False
                if data["score"] is None:
                    score = False
                if data["charge"] is None:
                    charge = False
                if data["retention_time"] is None:
                    rt = False
                if data["ion_mobility"] is None:
                    im_cv = False
                if data["additional_information"] is None:
                    additional_information = False
        return {
            "data_type": True,
            "completeness": True,
            "alpha_peptide": True,
            "alpha_modifications": modifications_a,
            "alpha_peptide_crosslink_position": True,
            "alpha_proteins": proteins_a,
            "alpha_proteins_crosslink_positions": xl_position_proteins_a,
            "alpha_proteins_peptide_positions": pep_position_proteins_a,
            "alpha_score": score_a,
            "alpha_decoy": decoy_a,
            "beta_peptide": True,
            "beta_modifications": modifications_b,
            "beta_peptide_crosslink_position": True,
            "beta_proteins": proteins_b,
            "beta_proteins_crosslink_positions": xl_position_proteins_b,
            "beta_proteins_peptide_positions": pep_position_proteins_b,
            "beta_score": score_b,
            "beta_decoy": decoy_b,
            "crosslink_type": True,
            "score": score,
            "spectrum_file": True,
            "scan_nr": True,
            "charge": charge,
            "retention_time": rt,
            "ion_mobility": im_cv,
            "additional_information": additional_information,
        }
    raise TypeError(
        f"Unknown data type {data_type}. Data type must be 'crosslink' or 'crosslink-spectrum-match'!"
    )
    return {"err": True}



#!/usr/bin/env python3

# 2025 (c) Micha Johannes Birklbauer
# https://github.com/michabirklbauer/
# micha.birklbauer@gmail.com

from __future__ import annotations

import pandas as pd

from ..data import check_input
from ..transform.util import get_available_keys
from .util import __get_filename

from typing import Optional
from typing import Dict
from typing import Any
from typing import List


def __xls_to_xlinkdb(
    xls: List[Dict[str, Any]],
    filename: Optional[str],
) -> pd.DataFrame:
    r"""Exports crosslinks to XLinkDB format.

    Parameters
    ----------
    xls : list of dict of str, any
        A list of crosslinks.
    filename : str, or None
        If not None, the data will be written to a file with the specified filename.

    Returns
    -------
    pd.DataFrame
        A pandas DataFrame in XLinkDB format.

    Notes
    -----
    This function should not be called directly, it is called from ``to_xlinkdb()``.
    """
    peptide_a = list()
    protein_a = list()
    labeled_position_a = list()
    peptide_b = list()
    protein_b = list()
    labeled_position_b = list()
    probability = list()
    for xl in xls:
        peptide_a.append(xl["alpha_peptide"])
        protein_a.append(xl["alpha_proteins"][0])
        labeled_position_a.append(xl["alpha_peptide_crosslink_position"] - 1)
        peptide_b.append(xl["beta_peptide"])
        protein_b.append(xl["beta_proteins"][0])
        labeled_position_b.append(xl["beta_peptide_crosslink_position"] - 1)
        probability.append(1)
    xlinkdb_df = pd.DataFrame(
        {
            "Peptide A": peptide_a,
            "Protein A": protein_a,
            "Labeled Position A": labeled_position_a,
            "Peptide B": peptide_b,
            "Protein B": protein_b,
            "Labeled Position B": labeled_position_b,
            "Probability": probability,
        }
    )
    if filename is not None:
        xlinkdb_df.to_csv(
            __get_filename(filename, "tsv"), sep="\t", header=False, index=False
        )
    return xlinkdb_df


def to_xlinkdb(
    crosslinks: List[Dict[str, Any]],
    filename: Optional[str],
) -> pd.DataFrame:
    r"""Exports a list of crosslinks to XLinkDB format.

    Exports a list of crosslinks to XLinkDB format. The tool XLinkDB is accessible
    via the link
    `xlinkdb.gs.washington.edu/xlinkdb <https://xlinkdb.gs.washington.edu/xlinkdb/index.php>`_.
    Requires that ``alpha_proteins`` and ``beta_proteins`` fields are set for all crosslinks.

    Parameters
    ----------
    crosslinks : list of dict of str, any
        A list of crosslinks.
    filename : str, or None
        If not None, the exported data will be written to a file with the specified filename.
        The filename should not contain a file extension and consist only of alpha-numeric
        characters (a-Z, 0-9).

    Returns
    -------
    pd.DataFrame
        A pandas DataFrame containing crosslinks in XLinkDB format.

    Raises
    ------
    TypeError
        If a wrong data type is provided.
    TypeError
        If 'crosslinks' parameter contains elements of mixed data type.
    ValueError
        If the filename contains any non-alpha-numeric characters.
    ValueError
        If the provided 'crosslinks' parameter contains no elements.
    RuntimeError
        If not all of the required information is present in the input data.

    Notes
    -----
    XLinkDB input format requires a column with probabilities that the crosslinks are correct. Since that is not available
    from most crosslink search engines, this is simply set to a constant ``1``.

    Examples
    --------
    >>> from pyXLMS.exporter import to_xlinkdb
    >>> from pyXLMS.parser import read
    >>> pr = read(
    ...     "data/xi/1perc_xl_boost_Links_xiFDR2.2.1.csv",
    ...     engine="xiSearch/xiFDR",
    ...     crosslinker="DSS",
    ... )
    >>> crosslinks = pr["crosslinks"]
    >>> to_xlinkdb(crosslinks, filename="crosslinksForXLinkDB")
                   Peptide A Protein A  Labeled Position A      Peptide B Protein B  Labeled Position B  Probability
    0            VVDELVKVMGR      Cas9                   6    VVDELVKVMGR      Cas9                   6            1
    1    MLASAGELQKGNELALPSK      Cas9                   9    VVDELVKVMGR      Cas9                   6            1
    2          MDGTEELLVKLNR      Cas9                   9  MDGTEELLVKLNR      Cas9                   9            1
    3           MTNFDKNLPNEK      Cas9                   5       SKLVSDFR      Cas9                   1            1
    4               DFQFYKVR      Cas9                   5    MIAKSEQEIGK      Cas9                   3            1
    ..                   ...       ...                 ...            ...       ...                 ...          ...
    222        LPKYSLFELENGR      Cas9                   2          SDKNR      Cas9                   2            1
    223               DKQSGK      Cas9                   1         DKQSGK      Cas9                   1            1
    224               AGFIKR      Cas9                   4   SDNVPSEEVVKK      Cas9                  10            1
    225                EKIEK      Cas9                   1          KVTVK      Cas9                   0            1
    226                LSKSR      Cas9                   2          LSKSR      Cas9                   2            1
    [227 rows x 7 columns]

    >>> from pyXLMS.exporter import to_xlinkdb
    >>> from pyXLMS.parser import read
    >>> pr = read(
    ...     "data/xi/1perc_xl_boost_Links_xiFDR2.2.1.csv",
    ...     engine="xiSearch/xiFDR",
    ...     crosslinker="DSS",
    ... )
    >>> crosslinks = pr["crosslinks"]
    >>> df = to_xlinkdb(crosslinks, filename=None)
    """
    _ok = check_input(crosslinks, "crosslinks", list, dict)
    _ok = check_input(filename, "filename", str) if filename is not None else True
    if filename is not None and not filename.isalnum():
        raise ValueError(
            "Parameter filename must only contain alpha-numeric characters and no file extension!"
        )
    if len(crosslinks) == 0:
        raise ValueError("Provided crosslinks contain no elements!")
    if "data_type" not in crosslinks[0] or crosslinks[0]["data_type"] != "crosslink":
        raise TypeError(
            "Unsupported data type for input crosslinks! Parameter crosslinks has to be a list of crosslinks!"
        )
    available_keys = get_available_keys(crosslinks)
    if not available_keys["alpha_proteins"] or not available_keys["beta_proteins"]:
        raise RuntimeError(
            "Can't export to XLinkDB because not all necessary information is available!"
        )
    return __xls_to_xlinkdb(crosslinks, filename)
